package cn.cloudcore.iprotect.utils;


import android.app.Activity;
import android.content.Context;
import android.content.ContextWrapper;
import android.os.Build;
import android.util.Base64;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

public class SecKeyUtils {

    public static int ERROR_FAILUE          = -1;   // 执行失败
    public static int ERROR_NO              = 0;
    public static int ERROR_PARAM           = 1;    // 参数错误
    public static int ERROR_SYSCALL         = 2;    // 系统调用
    public static int ERROR_MEMORY          = 3;    // 内存错误
    public static int ERROR_PADDING         = 4;    // 填充错误
    public static int ERROR_FILE_OPEN       = 5;    // 文件打开错误
    public static int ERROR_FILE_CONT       = 6;    // 文件内容错误

    public static int ERROR_NOT_INITIAL     = 10;   // 未初始化
    public static int ERROR_DATA_ABNORMA    = 11;   // 数据异常
    public static int ERROR_IS_FULL         = 12;   // 存储秘钥已满，最大存储格式为10个
    public static int ERROR_MODULUS_HMAC    = 13;   // 传入的公钥模数的hmac有问题
    public static int ERROR_NOT_FOUND       = 14;   // 没有找alias对应的秘钥
    public static int ERROR_KEY_CONTENT     = 15;   // 秘钥内容错误
    public static int ERROR_KEY_EXIST       = 16;   // 秘钥已经存在

    private String pubModulus = "";
    private String pubModulusHmac = "";


    public SecKeyUtils(Context context) {
        initNativeSecKeyUtils(getActivity(context), getSerialNumber());
    }

    private static String getSerialNumber() {
        String serial = "serial"; // default value
        
        String m_szDevIDShort = "35" + 
                Build.BOARD.length() % 10   + Build.BRAND.length() % 10 + 
                Build.CPU_ABI.length() % 10 + Build.DEVICE.length() % 10 + 
                Build.DISPLAY.length() % 10 + Build.HOST.length() % 10 + 
                Build.ID.length() % 10      + Build.MANUFACTURER.length() % 10 + 
                Build.MODEL.length() % 10   + Build.PRODUCT.length() % 10 + 
                Build.TAGS.length() % 10    + Build.TYPE.length() % 10 + 
                Build.USER.length() % 10; //13 位
        try {
            serial = android.os.Build.class.getField("SERIAL").get(null).toString();
            return serial;
        } catch (Exception exception) {}
        
        //使用硬件信息拼凑出来的15位号码
        return new UUID(m_szDevIDShort.hashCode(), serial.hashCode()).toString();
    }
    
    private static Activity getActivity(Context context) {
        while (!(context instanceof Activity) && context instanceof ContextWrapper) {
            context = ((ContextWrapper) context).getBaseContext();
        }

        if (context instanceof Activity) {
            return (Activity) context;
        }

        return (Activity) context;
    }

    /**
     * 设置公钥模数和对应的hmac值，如果不设置，获取密文。
     * @param modulus
     * @param hmac
     */
    public void setPublicModulus(String modulus, String hmac) {
        pubModulus = modulus;
        pubModulusHmac = hmac;
    }

    /**
     * 当前的秘钥个数
     * @return 当前的秘钥个数
     */
    public int getSecKeyNumber() {
        return getNativeSecKeyNumber();
    }

    /**
     * 获得当前所有的秘钥别名
     * @return 所有的秘钥别名
     */
    public List<String> getAllSecKeyAlias() {
        List<String> aliasList = new ArrayList<String>();

        String[] aliasArray = getNativeAllSecKeyAlias();
        if (aliasArray == null) return aliasList;

        for (int i = 0; i < aliasArray.length; i++) {
            aliasList.add(aliasArray[i]);
        }

        return aliasList;
    }

    /**
     * 删除已存在的秘钥
     * @param secKeyAlias
     */
    public void removeSecKey(String secKeyAlias) {
        if (secKeyAlias == null || secKeyAlias.length() == 0)
            return;

        removeNativeSecKey(secKeyAlias);
    }

    /**
     * 根据秘钥的别名，获取秘钥
     * @param secKeyAlias 秘钥的别名
     * @param secKeyFactor 16进制的因子，比如手势密码
     * @param timestamp 毫秒级别的时间戳
     * @return 加密后的秘钥,格式是json字符串{"KEY":"秘钥"}, 然后使用数字信封加密
     */
    public String getCipheredSecKey(String secKeyAlias, String secKeyFactor, String timestamp) {
        return getNativeCipheredSecKey(secKeyAlias, secKeyFactor, pubModulus, pubModulusHmac, timestamp);
    }

    /**
     * 根据秘钥的别名，产生秘钥，并以密文的格式返回
     * @param secKeyAlias    秘钥的别名
     * @return 返回结果 0为正确
     */
    public int generateSecKey(String secKeyAlias) {
        return generateNativeSecKey(secKeyAlias);
    }

    /**
     * 秘钥、挑战码，计算SHA1值，SHA1计算过程：
     *     1. secKeyFactor 为空（非手势） 直接计算seckey和挑战码的sha1
     *     2. secKeyFactor 不为null  先seckey和secKeyFactor计算sha1，结果在和挑战码计算sha1
     * @param  secKeyAlias     秘钥别名
     * @param  secKeyFactor    16进制的秘钥因子，比如手势密码，可以为null
     * @param  challengeCode   16进制的挑战码，通常是服务端产生
     * @return 16进制的SHA1值
     */
    public String SHA1(String secKeyAlias, String secKeyFactor, String challengeCode) {
        return nativeSHA1(secKeyAlias, secKeyFactor, challengeCode);
    }

    /**
     * 秘钥、挑战码，计算SHA1值，SHA1计算过程：
     *     1. secKeyFactor 为空（非手势） 直接计算seckey和挑战码的sha1
     *     2. secKeyFactor 不为null  先seckey和secKeyFactor计算sha1，结果在和挑战码计算sha1
     * @param  secKeyAlias     秘钥别名
     * @param  secKeyFactor    16进制的秘钥因子，比如手势密码，可以为null
     * @param  challengeCode   16进制的挑战码，通常是服务端产生
     * @param  timestamp       毫秒级别的时间戳
     * @return 密文形式,格式是json字符串{"KEY":"16进制的SHA1"}, 然后使用数字信封加密
     */
    public String SHA1WithCiphered(String secKeyAlias, String secKeyFactor, String challengeCode,
                                    String timestamp) {
        return nativeSHA1WithCiphered(secKeyAlias, secKeyFactor, challengeCode,
                    pubModulus, pubModulusHmac, timestamp);
    }

    /**
     * 如果未获得预期返回内容，可以通过lastError接口查看错误码
     * @return 错误码
     */

    public int lastError() {
        return nativeLastError();
    }

    /**
     * 16进制字符串转换为base64编码字符串
     * @param strHex
     * @return
     */
    public String HexToBase64(String strHex) {
        if (strHex == null || strHex.length() == 0)
            return null;

        if ((strHex.length() % 2) != 0)
            return null;

        byte[] bStr = HexString2Bytes(strHex);

        return Base64.encodeToString(bStr, Base64.NO_WRAP);
    }

    /**
     * base64编码字符串转换为16进制字符串
     * @param strBase64
     * @return
     */

    public String Base64ToHex(String strBase64) {
        if (strBase64 == null || strBase64.length() == 0)
            return null;

        byte[] bStr = Base64.decode(strBase64, Base64.NO_WRAP);
        if (bStr == null) return null;

        return Bytes2HexString(bStr);
    }

    private byte uniteBytes(byte src0, byte src1) {
        byte _b0 = Byte.decode("0x" + new String(new byte[]{src0})).byteValue();
        _b0 = (byte)(_b0 << 4);
        byte _b1 = Byte.decode("0x" + new String(new byte[]{src1})).byteValue();
        byte ret = (byte)(_b0 ^ _b1);
        return ret;
    }

    private byte[] HexString2Bytes(String src) {
        byte[] ret = new byte[src.length() / 2];
        byte[] tmp = src.getBytes();

        for(int i = 0; i < ret.length; ++i) {
            ret[i] = uniteBytes(tmp[i * 2], tmp[i * 2 + 1]);
        }

        return ret;
    }


    private  String Bytes2HexString(byte[] b) {
        String ret = "";

        for(int i = 0; i < b.length; ++i) {
            String hex = Integer.toHexString(b[i] & 255);
            if(hex.length() == 1) {
                hex = '0' + hex;
            }

            ret = ret + hex.toUpperCase();
        }

        return ret;
    }

    private native static void initNativeSecKeyUtils(Context context, String key);

    private native static int getNativeSecKeyNumber();
    private native static String[] getNativeAllSecKeyAlias();

    private native static String getNativeCipheredSecKey(String secKeyAlias, String secKeyFactor,
                                     String pubModulus, String pubModulusHmac, String timestamp);
    private native static int generateNativeSecKey(String secKeyAlias);
    private native static void   removeNativeSecKey(String secKeyAlias);
    private native static String nativeSHA1(String secKeyAlias, String secKeyFactor,
                                            String challengeCode);
    private native static String nativeSHA1WithCiphered(
                                String secKeyAlias, String secKeyFactor, String challengeCode,
                                String pubModulus, String pubModulusHmac, String timestamp);

    private native static int nativeLastError();

    static {
        System.loadLibrary(CBuildConfig.loadLibraryName);
    }
}
